import { TemplateLiteralToUnion } from '../template-literal/index.mjs';
import { Union } from '../union/index.mjs';
import { Never } from '../never/index.mjs';
import { ExtendsCheck, ExtendsResult } from '../extends/index.mjs';
import { CloneType } from '../clone/type.mjs';
import { ExcludeFromMappedResult } from './exclude-from-mapped-result.mjs';
// ------------------------------------------------------------------
// TypeGuard
// ------------------------------------------------------------------
// prettier-ignore
import { IsMappedResult, IsTemplateLiteral, IsUnion } from '../guard/type.mjs';
// prettier-ignore
function ExcludeResolve(L, R) {
    return (IsTemplateLiteral(L) ? ExcludeResolve(TemplateLiteralToUnion(L), R) :
        IsTemplateLiteral(R) ? ExcludeResolve(L, TemplateLiteralToUnion(R)) :
            IsUnion(L) ? (() => {
                const narrowed = L.anyOf.filter((inner) => ExtendsCheck(inner, R) === ExtendsResult.False);
                return (narrowed.length === 1 ? narrowed[0] : Union(narrowed));
            })() :
                ExtendsCheck(L, R) !== ExtendsResult.False ? Never() :
                    L);
}
/** `[Json]` Constructs a type by excluding from unionType all union members that are assignable to excludedMembers */
export function Exclude(unionType, excludedMembers, options = {}) {
    if (IsMappedResult(unionType)) {
        return ExcludeFromMappedResult(unionType, excludedMembers, options);
    }
    else {
        const E = ExcludeResolve(unionType, excludedMembers);
        return CloneType(E, options);
    }
}
