import { Any } from '../any/index.mjs';
import { Function as FunctionType } from '../function/index.mjs';
import { Number } from '../number/index.mjs';
import { String } from '../string/index.mjs';
import { Unknown } from '../unknown/index.mjs';
import { TemplateLiteralToUnion } from '../template-literal/index.mjs';
import { PatternNumberExact, PatternStringExact } from '../patterns/index.mjs';
import { Kind, Hint } from '../symbols/index.mjs';
import { TypeBoxError } from '../error/index.mjs';
import { TypeGuard, ValueGuard } from '../guard/index.mjs';
export class ExtendsResolverError extends TypeBoxError {
}
export var ExtendsResult;
(function (ExtendsResult) {
    ExtendsResult[ExtendsResult["Union"] = 0] = "Union";
    ExtendsResult[ExtendsResult["True"] = 1] = "True";
    ExtendsResult[ExtendsResult["False"] = 2] = "False";
})(ExtendsResult || (ExtendsResult = {}));
// ------------------------------------------------------------------
// IntoBooleanResult
// ------------------------------------------------------------------
// prettier-ignore
function IntoBooleanResult(result) {
    return result === ExtendsResult.False ? result : ExtendsResult.True;
}
// ------------------------------------------------------------------
// Throw
// ------------------------------------------------------------------
// prettier-ignore
function Throw(message) {
    throw new ExtendsResolverError(message);
}
// ------------------------------------------------------------------
// StructuralRight
// ------------------------------------------------------------------
// prettier-ignore
function IsStructuralRight(right) {
    return (TypeGuard.IsNever(right) ||
        TypeGuard.IsIntersect(right) ||
        TypeGuard.IsUnion(right) ||
        TypeGuard.IsUnknown(right) ||
        TypeGuard.IsAny(right));
}
// prettier-ignore
function StructuralRight(left, right) {
    return (TypeGuard.IsNever(right) ? TNeverRight(left, right) :
        TypeGuard.IsIntersect(right) ? TIntersectRight(left, right) :
            TypeGuard.IsUnion(right) ? TUnionRight(left, right) :
                TypeGuard.IsUnknown(right) ? TUnknownRight(left, right) :
                    TypeGuard.IsAny(right) ? TAnyRight(left, right) :
                        Throw('StructuralRight'));
}
// ------------------------------------------------------------------
// Any
// ------------------------------------------------------------------
// prettier-ignore
function TAnyRight(left, right) {
    return ExtendsResult.True;
}
// prettier-ignore
function TAny(left, right) {
    return (TypeGuard.IsIntersect(right) ? TIntersectRight(left, right) :
        (TypeGuard.IsUnion(right) && right.anyOf.some((schema) => TypeGuard.IsAny(schema) || TypeGuard.IsUnknown(schema))) ? ExtendsResult.True :
            TypeGuard.IsUnion(right) ? ExtendsResult.Union :
                TypeGuard.IsUnknown(right) ? ExtendsResult.True :
                    TypeGuard.IsAny(right) ? ExtendsResult.True :
                        ExtendsResult.Union);
}
// ------------------------------------------------------------------
// Array
// ------------------------------------------------------------------
// prettier-ignore
function TArrayRight(left, right) {
    return (TypeGuard.IsUnknown(left) ? ExtendsResult.False :
        TypeGuard.IsAny(left) ? ExtendsResult.Union :
            TypeGuard.IsNever(left) ? ExtendsResult.True :
                ExtendsResult.False);
}
// prettier-ignore
function TArray(left, right) {
    return (TypeGuard.IsObject(right) && IsObjectArrayLike(right) ? ExtendsResult.True :
        IsStructuralRight(right) ? StructuralRight(left, right) :
            !TypeGuard.IsArray(right) ? ExtendsResult.False :
                IntoBooleanResult(Visit(left.items, right.items)));
}
// ------------------------------------------------------------------
// AsyncIterator
// ------------------------------------------------------------------
// prettier-ignore
function TAsyncIterator(left, right) {
    return (IsStructuralRight(right) ? StructuralRight(left, right) :
        !TypeGuard.IsAsyncIterator(right) ? ExtendsResult.False :
            IntoBooleanResult(Visit(left.items, right.items)));
}
// ------------------------------------------------------------------
// BigInt
// ------------------------------------------------------------------
// prettier-ignore
function TBigInt(left, right) {
    return (IsStructuralRight(right) ? StructuralRight(left, right) :
        TypeGuard.IsObject(right) ? TObjectRight(left, right) :
            TypeGuard.IsRecord(right) ? TRecordRight(left, right) :
                TypeGuard.IsBigInt(right) ? ExtendsResult.True :
                    ExtendsResult.False);
}
// ------------------------------------------------------------------
// Boolean
// ------------------------------------------------------------------
// prettier-ignore
function TBooleanRight(left, right) {
    return (TypeGuard.IsLiteralBoolean(left) ? ExtendsResult.True :
        TypeGuard.IsBoolean(left) ? ExtendsResult.True :
            ExtendsResult.False);
}
// prettier-ignore
function TBoolean(left, right) {
    return (IsStructuralRight(right) ? StructuralRight(left, right) :
        TypeGuard.IsObject(right) ? TObjectRight(left, right) :
            TypeGuard.IsRecord(right) ? TRecordRight(left, right) :
                TypeGuard.IsBoolean(right) ? ExtendsResult.True :
                    ExtendsResult.False);
}
// ------------------------------------------------------------------
// Constructor
// ------------------------------------------------------------------
// prettier-ignore
function TConstructor(left, right) {
    return (IsStructuralRight(right) ? StructuralRight(left, right) :
        TypeGuard.IsObject(right) ? TObjectRight(left, right) :
            !TypeGuard.IsConstructor(right) ? ExtendsResult.False :
                left.parameters.length > right.parameters.length ? ExtendsResult.False :
                    (!left.parameters.every((schema, index) => IntoBooleanResult(Visit(right.parameters[index], schema)) === ExtendsResult.True)) ? ExtendsResult.False :
                        IntoBooleanResult(Visit(left.returns, right.returns)));
}
// ------------------------------------------------------------------
// Date
// ------------------------------------------------------------------
// prettier-ignore
function TDate(left, right) {
    return (IsStructuralRight(right) ? StructuralRight(left, right) :
        TypeGuard.IsObject(right) ? TObjectRight(left, right) :
            TypeGuard.IsRecord(right) ? TRecordRight(left, right) :
                TypeGuard.IsDate(right) ? ExtendsResult.True :
                    ExtendsResult.False);
}
// ------------------------------------------------------------------
// Function
// ------------------------------------------------------------------
// prettier-ignore
function TFunction(left, right) {
    return (IsStructuralRight(right) ? StructuralRight(left, right) :
        TypeGuard.IsObject(right) ? TObjectRight(left, right) :
            !TypeGuard.IsFunction(right) ? ExtendsResult.False :
                left.parameters.length > right.parameters.length ? ExtendsResult.False :
                    (!left.parameters.every((schema, index) => IntoBooleanResult(Visit(right.parameters[index], schema)) === ExtendsResult.True)) ? ExtendsResult.False :
                        IntoBooleanResult(Visit(left.returns, right.returns)));
}
// ------------------------------------------------------------------
// Integer
// ------------------------------------------------------------------
// prettier-ignore
function TIntegerRight(left, right) {
    return (TypeGuard.IsLiteral(left) && ValueGuard.IsNumber(left.const) ? ExtendsResult.True :
        TypeGuard.IsNumber(left) || TypeGuard.IsInteger(left) ? ExtendsResult.True :
            ExtendsResult.False);
}
// prettier-ignore
function TInteger(left, right) {
    return (TypeGuard.IsInteger(right) || TypeGuard.IsNumber(right) ? ExtendsResult.True :
        IsStructuralRight(right) ? StructuralRight(left, right) :
            TypeGuard.IsObject(right) ? TObjectRight(left, right) :
                TypeGuard.IsRecord(right) ? TRecordRight(left, right) :
                    ExtendsResult.False);
}
// ------------------------------------------------------------------
// Intersect
// ------------------------------------------------------------------
// prettier-ignore
function TIntersectRight(left, right) {
    return right.allOf.every((schema) => Visit(left, schema) === ExtendsResult.True)
        ? ExtendsResult.True
        : ExtendsResult.False;
}
// prettier-ignore
function TIntersect(left, right) {
    return left.allOf.some((schema) => Visit(schema, right) === ExtendsResult.True)
        ? ExtendsResult.True
        : ExtendsResult.False;
}
// ------------------------------------------------------------------
// Iterator
// ------------------------------------------------------------------
// prettier-ignore
function TIterator(left, right) {
    return (IsStructuralRight(right) ? StructuralRight(left, right) :
        !TypeGuard.IsIterator(right) ? ExtendsResult.False :
            IntoBooleanResult(Visit(left.items, right.items)));
}
// ------------------------------------------------------------------
// Literal
// ------------------------------------------------------------------
// prettier-ignore
function TLiteral(left, right) {
    return (TypeGuard.IsLiteral(right) && right.const === left.const ? ExtendsResult.True :
        IsStructuralRight(right) ? StructuralRight(left, right) :
            TypeGuard.IsObject(right) ? TObjectRight(left, right) :
                TypeGuard.IsRecord(right) ? TRecordRight(left, right) :
                    TypeGuard.IsString(right) ? TStringRight(left, right) :
                        TypeGuard.IsNumber(right) ? TNumberRight(left, right) :
                            TypeGuard.IsInteger(right) ? TIntegerRight(left, right) :
                                TypeGuard.IsBoolean(right) ? TBooleanRight(left, right) :
                                    ExtendsResult.False);
}
// ------------------------------------------------------------------
// Never
// ------------------------------------------------------------------
// prettier-ignore
function TNeverRight(left, right) {
    return ExtendsResult.False;
}
// prettier-ignore
function TNever(left, right) {
    return ExtendsResult.True;
}
// ------------------------------------------------------------------
// Not
// ------------------------------------------------------------------
// prettier-ignore
function UnwrapTNot(schema) {
    let [current, depth] = [schema, 0];
    while (true) {
        if (!TypeGuard.IsNot(current))
            break;
        current = current.not;
        depth += 1;
    }
    return depth % 2 === 0 ? current : Unknown();
}
// prettier-ignore
function TNot(left, right) {
    // TypeScript has no concept of negated types, and attempts to correctly check the negated
    // type at runtime would put TypeBox at odds with TypeScripts ability to statically infer
    // the type. Instead we unwrap to either unknown or T and continue evaluating.
    // prettier-ignore
    return (TypeGuard.IsNot(left) ? Visit(UnwrapTNot(left), right) :
        TypeGuard.IsNot(right) ? Visit(left, UnwrapTNot(right)) :
            Throw('Invalid fallthrough for Not'));
}
// ------------------------------------------------------------------
// Null
// ------------------------------------------------------------------
// prettier-ignore
function TNull(left, right) {
    return (IsStructuralRight(right) ? StructuralRight(left, right) :
        TypeGuard.IsObject(right) ? TObjectRight(left, right) :
            TypeGuard.IsRecord(right) ? TRecordRight(left, right) :
                TypeGuard.IsNull(right) ? ExtendsResult.True :
                    ExtendsResult.False);
}
// ------------------------------------------------------------------
// Number
// ------------------------------------------------------------------
// prettier-ignore
function TNumberRight(left, right) {
    return (TypeGuard.IsLiteralNumber(left) ? ExtendsResult.True :
        TypeGuard.IsNumber(left) || TypeGuard.IsInteger(left) ? ExtendsResult.True :
            ExtendsResult.False);
}
// prettier-ignore
function TNumber(left, right) {
    return (IsStructuralRight(right) ? StructuralRight(left, right) :
        TypeGuard.IsObject(right) ? TObjectRight(left, right) :
            TypeGuard.IsRecord(right) ? TRecordRight(left, right) :
                TypeGuard.IsInteger(right) || TypeGuard.IsNumber(right) ? ExtendsResult.True :
                    ExtendsResult.False);
}
// ------------------------------------------------------------------
// Object
// ------------------------------------------------------------------
// prettier-ignore
function IsObjectPropertyCount(schema, count) {
    return Object.getOwnPropertyNames(schema.properties).length === count;
}
// prettier-ignore
function IsObjectStringLike(schema) {
    return IsObjectArrayLike(schema);
}
// prettier-ignore
function IsObjectSymbolLike(schema) {
    return IsObjectPropertyCount(schema, 0) || (IsObjectPropertyCount(schema, 1) && 'description' in schema.properties && TypeGuard.IsUnion(schema.properties.description) && schema.properties.description.anyOf.length === 2 && ((TypeGuard.IsString(schema.properties.description.anyOf[0]) &&
        TypeGuard.IsUndefined(schema.properties.description.anyOf[1])) || (TypeGuard.IsString(schema.properties.description.anyOf[1]) &&
        TypeGuard.IsUndefined(schema.properties.description.anyOf[0]))));
}
// prettier-ignore
function IsObjectNumberLike(schema) {
    return IsObjectPropertyCount(schema, 0);
}
// prettier-ignore
function IsObjectBooleanLike(schema) {
    return IsObjectPropertyCount(schema, 0);
}
// prettier-ignore
function IsObjectBigIntLike(schema) {
    return IsObjectPropertyCount(schema, 0);
}
// prettier-ignore
function IsObjectDateLike(schema) {
    return IsObjectPropertyCount(schema, 0);
}
// prettier-ignore
function IsObjectUint8ArrayLike(schema) {
    return IsObjectArrayLike(schema);
}
// prettier-ignore
function IsObjectFunctionLike(schema) {
    const length = Number();
    return IsObjectPropertyCount(schema, 0) || (IsObjectPropertyCount(schema, 1) && 'length' in schema.properties && IntoBooleanResult(Visit(schema.properties['length'], length)) === ExtendsResult.True);
}
// prettier-ignore
function IsObjectConstructorLike(schema) {
    return IsObjectPropertyCount(schema, 0);
}
// prettier-ignore
function IsObjectArrayLike(schema) {
    const length = Number();
    return IsObjectPropertyCount(schema, 0) || (IsObjectPropertyCount(schema, 1) && 'length' in schema.properties && IntoBooleanResult(Visit(schema.properties['length'], length)) === ExtendsResult.True);
}
// prettier-ignore
function IsObjectPromiseLike(schema) {
    const then = FunctionType([Any()], Any());
    return IsObjectPropertyCount(schema, 0) || (IsObjectPropertyCount(schema, 1) && 'then' in schema.properties && IntoBooleanResult(Visit(schema.properties['then'], then)) === ExtendsResult.True);
}
// ------------------------------------------------------------------
// Property
// ------------------------------------------------------------------
// prettier-ignore
function Property(left, right) {
    return (Visit(left, right) === ExtendsResult.False ? ExtendsResult.False :
        TypeGuard.IsOptional(left) && !TypeGuard.IsOptional(right) ? ExtendsResult.False :
            ExtendsResult.True);
}
// prettier-ignore
function TObjectRight(left, right) {
    return (TypeGuard.IsUnknown(left) ? ExtendsResult.False :
        TypeGuard.IsAny(left) ? ExtendsResult.Union : (TypeGuard.IsNever(left) ||
            (TypeGuard.IsLiteralString(left) && IsObjectStringLike(right)) ||
            (TypeGuard.IsLiteralNumber(left) && IsObjectNumberLike(right)) ||
            (TypeGuard.IsLiteralBoolean(left) && IsObjectBooleanLike(right)) ||
            (TypeGuard.IsSymbol(left) && IsObjectSymbolLike(right)) ||
            (TypeGuard.IsBigInt(left) && IsObjectBigIntLike(right)) ||
            (TypeGuard.IsString(left) && IsObjectStringLike(right)) ||
            (TypeGuard.IsSymbol(left) && IsObjectSymbolLike(right)) ||
            (TypeGuard.IsNumber(left) && IsObjectNumberLike(right)) ||
            (TypeGuard.IsInteger(left) && IsObjectNumberLike(right)) ||
            (TypeGuard.IsBoolean(left) && IsObjectBooleanLike(right)) ||
            (TypeGuard.IsUint8Array(left) && IsObjectUint8ArrayLike(right)) ||
            (TypeGuard.IsDate(left) && IsObjectDateLike(right)) ||
            (TypeGuard.IsConstructor(left) && IsObjectConstructorLike(right)) ||
            (TypeGuard.IsFunction(left) && IsObjectFunctionLike(right))) ? ExtendsResult.True :
            (TypeGuard.IsRecord(left) && TypeGuard.IsString(RecordKey(left))) ? (() => {
                // When expressing a Record with literal key values, the Record is converted into a Object with
                // the Hint assigned as `Record`. This is used to invert the extends logic.
                return right[Hint] === 'Record' ? ExtendsResult.True : ExtendsResult.False;
            })() :
                (TypeGuard.IsRecord(left) && TypeGuard.IsNumber(RecordKey(left))) ? (() => {
                    return IsObjectPropertyCount(right, 0) ? ExtendsResult.True : ExtendsResult.False;
                })() :
                    ExtendsResult.False);
}
// prettier-ignore
function TObject(left, right) {
    return (IsStructuralRight(right) ? StructuralRight(left, right) :
        TypeGuard.IsRecord(right) ? TRecordRight(left, right) :
            !TypeGuard.IsObject(right) ? ExtendsResult.False :
                (() => {
                    for (const key of Object.getOwnPropertyNames(right.properties)) {
                        if (!(key in left.properties) && !TypeGuard.IsOptional(right.properties[key])) {
                            return ExtendsResult.False;
                        }
                        if (TypeGuard.IsOptional(right.properties[key])) {
                            return ExtendsResult.True;
                        }
                        if (Property(left.properties[key], right.properties[key]) === ExtendsResult.False) {
                            return ExtendsResult.False;
                        }
                    }
                    return ExtendsResult.True;
                })());
}
// ------------------------------------------------------------------
// Promise
// ------------------------------------------------------------------
// prettier-ignore
function TPromise(left, right) {
    return (IsStructuralRight(right) ? StructuralRight(left, right) :
        TypeGuard.IsObject(right) && IsObjectPromiseLike(right) ? ExtendsResult.True :
            !TypeGuard.IsPromise(right) ? ExtendsResult.False :
                IntoBooleanResult(Visit(left.item, right.item)));
}
// ------------------------------------------------------------------
// Record
// ------------------------------------------------------------------
// prettier-ignore
function RecordKey(schema) {
    return (PatternNumberExact in schema.patternProperties ? Number() :
        PatternStringExact in schema.patternProperties ? String() :
            Throw('Unknown record key pattern'));
}
// prettier-ignore
function RecordValue(schema) {
    return (PatternNumberExact in schema.patternProperties ? schema.patternProperties[PatternNumberExact] :
        PatternStringExact in schema.patternProperties ? schema.patternProperties[PatternStringExact] :
            Throw('Unable to get record value schema'));
}
// prettier-ignore
function TRecordRight(left, right) {
    const [Key, Value] = [RecordKey(right), RecordValue(right)];
    return ((TypeGuard.IsLiteralString(left) && TypeGuard.IsNumber(Key) && IntoBooleanResult(Visit(left, Value)) === ExtendsResult.True) ? ExtendsResult.True :
        TypeGuard.IsUint8Array(left) && TypeGuard.IsNumber(Key) ? Visit(left, Value) :
            TypeGuard.IsString(left) && TypeGuard.IsNumber(Key) ? Visit(left, Value) :
                TypeGuard.IsArray(left) && TypeGuard.IsNumber(Key) ? Visit(left, Value) :
                    TypeGuard.IsObject(left) ? (() => {
                        for (const key of Object.getOwnPropertyNames(left.properties)) {
                            if (Property(Value, left.properties[key]) === ExtendsResult.False) {
                                return ExtendsResult.False;
                            }
                        }
                        return ExtendsResult.True;
                    })() :
                        ExtendsResult.False);
}
// prettier-ignore
function TRecord(left, right) {
    return (IsStructuralRight(right) ? StructuralRight(left, right) :
        TypeGuard.IsObject(right) ? TObjectRight(left, right) :
            !TypeGuard.IsRecord(right) ? ExtendsResult.False :
                Visit(RecordValue(left), RecordValue(right)));
}
// ------------------------------------------------------------------
// String
// ------------------------------------------------------------------
// prettier-ignore
function TStringRight(left, right) {
    return (TypeGuard.IsLiteral(left) && ValueGuard.IsString(left.const) ? ExtendsResult.True :
        TypeGuard.IsString(left) ? ExtendsResult.True :
            ExtendsResult.False);
}
// prettier-ignore
function TString(left, right) {
    return (IsStructuralRight(right) ? StructuralRight(left, right) :
        TypeGuard.IsObject(right) ? TObjectRight(left, right) :
            TypeGuard.IsRecord(right) ? TRecordRight(left, right) :
                TypeGuard.IsString(right) ? ExtendsResult.True :
                    ExtendsResult.False);
}
// ------------------------------------------------------------------
// Symbol
// ------------------------------------------------------------------
// prettier-ignore
function TSymbol(left, right) {
    return (IsStructuralRight(right) ? StructuralRight(left, right) :
        TypeGuard.IsObject(right) ? TObjectRight(left, right) :
            TypeGuard.IsRecord(right) ? TRecordRight(left, right) :
                TypeGuard.IsSymbol(right) ? ExtendsResult.True :
                    ExtendsResult.False);
}
// ------------------------------------------------------------------
// TemplateLiteral
// ------------------------------------------------------------------
// prettier-ignore
function TTemplateLiteral(left, right) {
    // TemplateLiteral types are resolved to either unions for finite expressions or string
    // for infinite expressions. Here we call to TemplateLiteralResolver to resolve for
    // either type and continue evaluating.
    return (TypeGuard.IsTemplateLiteral(left) ? Visit(TemplateLiteralToUnion(left), right) :
        TypeGuard.IsTemplateLiteral(right) ? Visit(left, TemplateLiteralToUnion(right)) :
            Throw('Invalid fallthrough for TemplateLiteral'));
}
// ------------------------------------------------------------------
// Tuple
// ------------------------------------------------------------------
// prettier-ignore
function IsArrayOfTuple(left, right) {
    return (TypeGuard.IsArray(right) &&
        left.items !== undefined &&
        left.items.every((schema) => Visit(schema, right.items) === ExtendsResult.True));
}
// prettier-ignore
function TTupleRight(left, right) {
    return (TypeGuard.IsNever(left) ? ExtendsResult.True :
        TypeGuard.IsUnknown(left) ? ExtendsResult.False :
            TypeGuard.IsAny(left) ? ExtendsResult.Union :
                ExtendsResult.False);
}
// prettier-ignore
function TTuple(left, right) {
    return (IsStructuralRight(right) ? StructuralRight(left, right) :
        TypeGuard.IsObject(right) && IsObjectArrayLike(right) ? ExtendsResult.True :
            TypeGuard.IsArray(right) && IsArrayOfTuple(left, right) ? ExtendsResult.True :
                !TypeGuard.IsTuple(right) ? ExtendsResult.False :
                    (ValueGuard.IsUndefined(left.items) && !ValueGuard.IsUndefined(right.items)) || (!ValueGuard.IsUndefined(left.items) && ValueGuard.IsUndefined(right.items)) ? ExtendsResult.False :
                        (ValueGuard.IsUndefined(left.items) && !ValueGuard.IsUndefined(right.items)) ? ExtendsResult.True :
                            left.items.every((schema, index) => Visit(schema, right.items[index]) === ExtendsResult.True) ? ExtendsResult.True :
                                ExtendsResult.False);
}
// ------------------------------------------------------------------
// Uint8Array
// ------------------------------------------------------------------
// prettier-ignore
function TUint8Array(left, right) {
    return (IsStructuralRight(right) ? StructuralRight(left, right) :
        TypeGuard.IsObject(right) ? TObjectRight(left, right) :
            TypeGuard.IsRecord(right) ? TRecordRight(left, right) :
                TypeGuard.IsUint8Array(right) ? ExtendsResult.True :
                    ExtendsResult.False);
}
// ------------------------------------------------------------------
// Undefined
// ------------------------------------------------------------------
// prettier-ignore
function TUndefined(left, right) {
    return (IsStructuralRight(right) ? StructuralRight(left, right) :
        TypeGuard.IsObject(right) ? TObjectRight(left, right) :
            TypeGuard.IsRecord(right) ? TRecordRight(left, right) :
                TypeGuard.IsVoid(right) ? VoidRight(left, right) :
                    TypeGuard.IsUndefined(right) ? ExtendsResult.True :
                        ExtendsResult.False);
}
// ------------------------------------------------------------------
// Union
// ------------------------------------------------------------------
// prettier-ignore
function TUnionRight(left, right) {
    return right.anyOf.some((schema) => Visit(left, schema) === ExtendsResult.True)
        ? ExtendsResult.True
        : ExtendsResult.False;
}
// prettier-ignore
function TUnion(left, right) {
    return left.anyOf.every((schema) => Visit(schema, right) === ExtendsResult.True)
        ? ExtendsResult.True
        : ExtendsResult.False;
}
// ------------------------------------------------------------------
// Unknown
// ------------------------------------------------------------------
// prettier-ignore
function TUnknownRight(left, right) {
    return ExtendsResult.True;
}
// prettier-ignore
function TUnknown(left, right) {
    return (TypeGuard.IsNever(right) ? TNeverRight(left, right) :
        TypeGuard.IsIntersect(right) ? TIntersectRight(left, right) :
            TypeGuard.IsUnion(right) ? TUnionRight(left, right) :
                TypeGuard.IsAny(right) ? TAnyRight(left, right) :
                    TypeGuard.IsString(right) ? TStringRight(left, right) :
                        TypeGuard.IsNumber(right) ? TNumberRight(left, right) :
                            TypeGuard.IsInteger(right) ? TIntegerRight(left, right) :
                                TypeGuard.IsBoolean(right) ? TBooleanRight(left, right) :
                                    TypeGuard.IsArray(right) ? TArrayRight(left, right) :
                                        TypeGuard.IsTuple(right) ? TTupleRight(left, right) :
                                            TypeGuard.IsObject(right) ? TObjectRight(left, right) :
                                                TypeGuard.IsUnknown(right) ? ExtendsResult.True :
                                                    ExtendsResult.False);
}
// ------------------------------------------------------------------
// Void
// ------------------------------------------------------------------
// prettier-ignore
function VoidRight(left, right) {
    return (TypeGuard.IsUndefined(left) ? ExtendsResult.True :
        TypeGuard.IsUndefined(left) ? ExtendsResult.True :
            ExtendsResult.False);
}
// prettier-ignore
function TVoid(left, right) {
    return (TypeGuard.IsIntersect(right) ? TIntersectRight(left, right) :
        TypeGuard.IsUnion(right) ? TUnionRight(left, right) :
            TypeGuard.IsUnknown(right) ? TUnknownRight(left, right) :
                TypeGuard.IsAny(right) ? TAnyRight(left, right) :
                    TypeGuard.IsObject(right) ? TObjectRight(left, right) :
                        TypeGuard.IsVoid(right) ? ExtendsResult.True :
                            ExtendsResult.False);
}
// prettier-ignore
function Visit(left, right) {
    return (
    // resolvable
    (TypeGuard.IsTemplateLiteral(left) || TypeGuard.IsTemplateLiteral(right)) ? TTemplateLiteral(left, right) :
        (TypeGuard.IsNot(left) || TypeGuard.IsNot(right)) ? TNot(left, right) :
            // standard
            TypeGuard.IsAny(left) ? TAny(left, right) :
                TypeGuard.IsArray(left) ? TArray(left, right) :
                    TypeGuard.IsBigInt(left) ? TBigInt(left, right) :
                        TypeGuard.IsBoolean(left) ? TBoolean(left, right) :
                            TypeGuard.IsAsyncIterator(left) ? TAsyncIterator(left, right) :
                                TypeGuard.IsConstructor(left) ? TConstructor(left, right) :
                                    TypeGuard.IsDate(left) ? TDate(left, right) :
                                        TypeGuard.IsFunction(left) ? TFunction(left, right) :
                                            TypeGuard.IsInteger(left) ? TInteger(left, right) :
                                                TypeGuard.IsIntersect(left) ? TIntersect(left, right) :
                                                    TypeGuard.IsIterator(left) ? TIterator(left, right) :
                                                        TypeGuard.IsLiteral(left) ? TLiteral(left, right) :
                                                            TypeGuard.IsNever(left) ? TNever(left, right) :
                                                                TypeGuard.IsNull(left) ? TNull(left, right) :
                                                                    TypeGuard.IsNumber(left) ? TNumber(left, right) :
                                                                        TypeGuard.IsObject(left) ? TObject(left, right) :
                                                                            TypeGuard.IsRecord(left) ? TRecord(left, right) :
                                                                                TypeGuard.IsString(left) ? TString(left, right) :
                                                                                    TypeGuard.IsSymbol(left) ? TSymbol(left, right) :
                                                                                        TypeGuard.IsTuple(left) ? TTuple(left, right) :
                                                                                            TypeGuard.IsPromise(left) ? TPromise(left, right) :
                                                                                                TypeGuard.IsUint8Array(left) ? TUint8Array(left, right) :
                                                                                                    TypeGuard.IsUndefined(left) ? TUndefined(left, right) :
                                                                                                        TypeGuard.IsUnion(left) ? TUnion(left, right) :
                                                                                                            TypeGuard.IsUnknown(left) ? TUnknown(left, right) :
                                                                                                                TypeGuard.IsVoid(left) ? TVoid(left, right) :
                                                                                                                    Throw(`Unknown left type operand '${left[Kind]}'`));
}
export function ExtendsCheck(left, right) {
    return Visit(left, right);
}
