/** Returns true if element S is in the set of T */
// prettier-ignore
export function OperatorIncludes(T, S) {
    const [L, ...R] = T;
    return (T.length > 0
        ? S === L
            ? true
            : OperatorIncludes(R, S)
        : false);
}
/** Returns true if T is a subset of S */
export function OperatorSubset(T, S) {
    const [L, ...R] = T;
    // prettier-ignore
    return (T.length > 0
        ? OperatorIncludes(S, L) === true
            ? OperatorSubset(R, S)
            : false
        : true);
}
/** Returns a distinct set of elements */
export function OperatorDistinct(T, Acc = []) {
    const [L, ...R] = T;
    // prettier-ignore
    return (T.length > 0 ?
        (OperatorIncludes(Acc, L) === false
            ? OperatorDistinct(R, [...Acc, L])
            : OperatorDistinct(R, [...Acc])) : Acc);
}
/** Returns the Intersect of the given sets */
export function OperatorIntersect(T, S) {
    const [L, ...R] = T;
    // prettier-ignore
    return (T.length > 0
        ? OperatorIncludes(S, L) === true
            ? [L, ...OperatorIntersect(R, S)]
            : [...OperatorIntersect(R, S)]
        : []);
}
/** Returns the Union of the given sets */
export function OperatorUnion(T, S, Acc = S) {
    const [L, ...R] = T;
    // prettier-ignore
    return (T.length > 0
        ? OperatorUnion(R, S, [...Acc, L])
        : Acc);
}
/** Returns the Complement by omitting elements in T that are in S */
// prettier-ignore
export function OperatorComplement(T, S, Acc = []) {
    const [L, ...R] = T;
    return (T.length > 0
        ? OperatorIncludes(S, L) === true
            ? OperatorComplement(R, S, [...Acc])
            : OperatorComplement(R, S, [L, ...Acc])
        : Acc);
}
/** Returns the Intersect of multiple sets */
// prettier-ignore
export function OperatorIntersectMany(T, Acc = []) {
    return (T.length === 1
        ? T[0]
        : (() => {
            const [L, ...R] = T;
            return (L.length > 0
                ? OperatorIntersect(L, OperatorIntersectMany(R))
                : []);
        })());
}
/** Returns the Union of multiple sets */
export function OperatorUnionMany(T, Acc = []) {
    const [L, ...R] = T;
    // prettier-ignore
    return (T.length > 0
        ? OperatorUnionMany(R, OperatorUnion(Acc, L))
        : Acc);
}
