import { Intersect } from '../intersect/index.mjs';
import { Union } from '../union/index.mjs';
import { Object } from '../object/index.mjs';
import { OptionalKind, TransformKind } from '../symbols/index.mjs';
import { CloneType } from '../clone/type.mjs';
import { Discard } from '../discard/index.mjs';
import { RequiredFromMappedResult } from './required-from-mapped-result.mjs';
// ------------------------------------------------------------------
// TypeGuard
// ------------------------------------------------------------------
import { IsMappedResult, IsIntersect, IsUnion, IsObject } from '../guard/type.mjs';
// prettier-ignore
function FromIntersect(T, Acc = []) {
    const [L, ...R] = T;
    return (T.length > 0
        ? FromIntersect(R, [...Acc, RequiredResolve(L)])
        : Acc);
}
// prettier-ignore
function FromUnion(T, Acc = []) {
    const [L, ...R] = T;
    return (T.length > 0
        ? FromUnion(R, [...Acc, RequiredResolve(L)])
        : Acc);
}
// prettier-ignore
function FromProperties(T) {
    return globalThis.Object.getOwnPropertyNames(T).reduce((Acc, K) => {
        return { ...Acc, [K]: Discard(T[K], [OptionalKind]) };
    }, {});
}
// prettier-ignore
function RequiredResolve(T) {
    return (IsIntersect(T) ? Intersect(FromIntersect(T.allOf)) :
        IsUnion(T) ? Union(FromUnion(T.anyOf)) :
            IsObject(T) ? Object(FromProperties(T.properties)) :
                Object({}));
}
/** `[Json]` Constructs a type where all properties are required */
export function Required(T, options = {}) {
    if (IsMappedResult(T)) {
        return RequiredFromMappedResult(T, options);
    }
    else {
        const D = Discard(T, [TransformKind, '$id', 'required']);
        const R = CloneType(RequiredResolve(T), options);
        return { ...D, ...R };
    }
}
