import { OptionalKind } from '../symbols/index.mjs';
import { CloneType } from '../clone/type.mjs';
import { Discard } from '../discard/index.mjs';
import { Never } from '../never/index.mjs';
import { Optional } from '../optional/index.mjs';
import { UnionCreate } from './union-create.mjs';
// ------------------------------------------------------------------
// TypeGuard
// ------------------------------------------------------------------
// prettier-ignore
import { IsOptional } from '../guard/type.mjs';
// prettier-ignore
function IsUnionOptional(T) {
    const [L, ...R] = T;
    return (T.length > 0
        ? IsOptional(L)
            ? true
            : IsUnionOptional(R)
        : false);
}
// prettier-ignore
function RemoveOptionalFromRest(T, Acc = []) {
    const [L, ...R] = T;
    return (T.length > 0
        ? IsOptional(L)
            ? RemoveOptionalFromRest(R, [...Acc, RemoveOptionalFromType(L)])
            : RemoveOptionalFromRest(R, [...Acc, L])
        : Acc);
}
// prettier-ignore
function RemoveOptionalFromType(T) {
    return (Discard(T, [OptionalKind]));
}
// prettier-ignore
function ResolveUnion(T, options) {
    return (IsUnionOptional(T)
        ? Optional(UnionCreate(RemoveOptionalFromRest(T), options))
        : UnionCreate(RemoveOptionalFromRest(T), options));
}
/** `[Json]` Creates an evaluated Union type */
export function UnionEvaluated(T, options = {}) {
    // prettier-ignore
    return (T.length === 0 ? Never(options) :
        T.length === 1 ? CloneType(T[0], options) :
            ResolveUnion(T, options));
}
