"use strict";

Object.defineProperty(exports, "__esModule", { value: true });
exports.KeyOfPattern = exports.KeyOfPropertyKeys = void 0;
const index_1 = require("../helpers/index");
const index_2 = require("../operators/index");
// ------------------------------------------------------------------
// TypeGuard
// ------------------------------------------------------------------
// prettier-ignore
const type_1 = require("../guard/type");
// prettier-ignore
function FromRest(T, Acc = []) {
    const [L, ...R] = T;
    return (T.length > 0
        ? FromRest(R, [...Acc, KeyOfPropertyKeys(L)])
        : Acc);
}
// prettier-ignore
function FromIntersect(T) {
    const C = FromRest(T);
    const R = (0, index_2.OperatorUnionMany)(C);
    return R;
}
// prettier-ignore
function FromUnion(T) {
    const C = FromRest(T);
    const R = (0, index_2.OperatorIntersectMany)(C);
    return R;
}
// prettier-ignore
function FromTuple(T, I = '0', Acc = []) {
    const [_, ...R] = T;
    return (T.length > 0
        ? FromTuple(R, (0, index_1.Increment)(I), [...Acc, I])
        : Acc);
}
// prettier-ignore
function FromArray(_) {
    return (['[number]']);
}
// prettier-ignore
function FromProperties(T) {
    return (globalThis.Object.getOwnPropertyNames(T));
}
// ------------------------------------------------------------------
// FromPatternProperties
// ------------------------------------------------------------------
// prettier-ignore
function FromPatternProperties(patternProperties) {
    if (!includePatternProperties)
        return [];
    const patternPropertyKeys = globalThis.Object.getOwnPropertyNames(patternProperties);
    return patternPropertyKeys.map(key => {
        return (key[0] === '^' && key[key.length - 1] === '$')
            ? key.slice(1, key.length - 1)
            : key;
    });
}
/** Returns a tuple of PropertyKeys derived from the given TSchema. */
// prettier-ignore
function KeyOfPropertyKeys(T) {
    return ((0, type_1.IsIntersect)(T) ? FromIntersect(T.allOf) :
        (0, type_1.IsUnion)(T) ? FromUnion(T.anyOf) :
            (0, type_1.IsTuple)(T) ? FromTuple(T.items ?? []) :
                (0, type_1.IsArray)(T) ? FromArray(T.items) :
                    (0, type_1.IsObject)(T) ? FromProperties(T.properties) :
                        (0, type_1.IsRecord)(T) ? FromPatternProperties(T.patternProperties) :
                            []);
}
exports.KeyOfPropertyKeys = KeyOfPropertyKeys;
// ----------------------------------------------------------------
// KeyOfPattern
// ----------------------------------------------------------------
let includePatternProperties = false;
/** Returns a regular expression pattern derived from the given TSchema */
function KeyOfPattern(schema) {
    includePatternProperties = true;
    const keys = KeyOfPropertyKeys(schema);
    includePatternProperties = false;
    const pattern = keys.map((key) => `(${key})`);
    return `^(${pattern.join('|')})$`;
}
exports.KeyOfPattern = KeyOfPattern;
