"use strict";

Object.defineProperty(exports, "__esModule", { value: true });
exports.OperatorUnionMany = exports.OperatorIntersectMany = exports.OperatorComplement = exports.OperatorUnion = exports.OperatorIntersect = exports.OperatorDistinct = exports.OperatorSubset = exports.OperatorIncludes = void 0;
/** Returns true if element S is in the set of T */
// prettier-ignore
function OperatorIncludes(T, S) {
    const [L, ...R] = T;
    return (T.length > 0
        ? S === L
            ? true
            : OperatorIncludes(R, S)
        : false);
}
exports.OperatorIncludes = OperatorIncludes;
/** Returns true if T is a subset of S */
function OperatorSubset(T, S) {
    const [L, ...R] = T;
    // prettier-ignore
    return (T.length > 0
        ? OperatorIncludes(S, L) === true
            ? OperatorSubset(R, S)
            : false
        : true);
}
exports.OperatorSubset = OperatorSubset;
/** Returns a distinct set of elements */
function OperatorDistinct(T, Acc = []) {
    const [L, ...R] = T;
    // prettier-ignore
    return (T.length > 0 ?
        (OperatorIncludes(Acc, L) === false
            ? OperatorDistinct(R, [...Acc, L])
            : OperatorDistinct(R, [...Acc])) : Acc);
}
exports.OperatorDistinct = OperatorDistinct;
/** Returns the Intersect of the given sets */
function OperatorIntersect(T, S) {
    const [L, ...R] = T;
    // prettier-ignore
    return (T.length > 0
        ? OperatorIncludes(S, L) === true
            ? [L, ...OperatorIntersect(R, S)]
            : [...OperatorIntersect(R, S)]
        : []);
}
exports.OperatorIntersect = OperatorIntersect;
/** Returns the Union of the given sets */
function OperatorUnion(T, S, Acc = S) {
    const [L, ...R] = T;
    // prettier-ignore
    return (T.length > 0
        ? OperatorUnion(R, S, [...Acc, L])
        : Acc);
}
exports.OperatorUnion = OperatorUnion;
/** Returns the Complement by omitting elements in T that are in S */
// prettier-ignore
function OperatorComplement(T, S, Acc = []) {
    const [L, ...R] = T;
    return (T.length > 0
        ? OperatorIncludes(S, L) === true
            ? OperatorComplement(R, S, [...Acc])
            : OperatorComplement(R, S, [L, ...Acc])
        : Acc);
}
exports.OperatorComplement = OperatorComplement;
/** Returns the Intersect of multiple sets */
// prettier-ignore
function OperatorIntersectMany(T, Acc = []) {
    return (T.length === 1
        ? T[0]
        : (() => {
            const [L, ...R] = T;
            return (L.length > 0
                ? OperatorIntersect(L, OperatorIntersectMany(R))
                : []);
        })());
}
exports.OperatorIntersectMany = OperatorIntersectMany;
/** Returns the Union of multiple sets */
function OperatorUnionMany(T, Acc = []) {
    const [L, ...R] = T;
    // prettier-ignore
    return (T.length > 0
        ? OperatorUnionMany(R, OperatorUnion(Acc, L))
        : Acc);
}
exports.OperatorUnionMany = OperatorUnionMany;
